"""
CHIPSEQ_

COMMON               SE                 PE                 NARROW               BROAD     TRIM_SE       TRIM_PE

bigwig
                     fastx_statistics   fastx_statistics_upstream
                                        fastx_statistics_downstream
bowtie_log
iaintersect_log
iaintersect_result
atdp_log
atdp_result
samtools_rmdup_log
bambai_pair
macs2_called_peaks
                                                           macs2_narrow_peaks
                                                           macs2_peak_summits
                                                                                macs2_broad_peaks
                                                                                macs2_gapped_peak
macs2_moder_r
macs2_log
get_stat_log
macs2_fragment_stat
                                                                                          trim_report
                                                                                                        trim_report_upstream:
                                                                                                        trim_report_downstream:
"""

COMMON = """
  {{
    "bowtie_log": {{
      "location": "{raw_data}/{uid}/{uid}.bw",
      "class": "File"
    }},
    "get_stat_log": {{
      "location": "{raw_data}/{uid}/{uid}.stat",
      "class": "File"
    }}
  }}
"""

CHIP = """
  {{
    "iaintersect_log": {{
      "location": "{raw_data}/{uid}/{uid}_macs_peaks_iaintersect.log",
      "class": "File"
    }},
    "iaintersect_result": {{
      "location": "{raw_data}/{uid}/{uid}_macs_peaks_iaintersect.tsv",
      "class": "File"
    }},
    "atdp_log": {{
      "location": "{raw_data}/{uid}/{uid}_atdp.log",
      "class": "File"
    }},
    "atdp_result": {{
      "location": "{raw_data}/{uid}/{uid}_atdp.tsv",
      "class": "File"
    }},
    "samtools_rmdup_log": {{
      "location": "{raw_data}/{uid}/{uid}.rmdup",
      "class": "File"
    }},
    "macs2_called_peaks": {{
      "location": "{raw_data}/{uid}/{uid}_macs_peaks.xls",
      "class": "File"
    }},
    "macs2_log": {{
      "location": "{raw_data}/{uid}/{uid}_macs.log",
      "class": "File"
    }},
    "macs2_fragment_stat": {{
      "location": "{raw_data}/{uid}/{uid}_fragment_stat.tsv",
      "class": "File"
    }}
  }}
"""

RNA = """
  {{
    "star_final_log": {{
      "location": "{raw_data}/{uid}/{uid}.Log.final.out",
      "class": "File"
    }},
    "star_out_log": {{
      "location": "{raw_data}/{uid}/{uid}.Log.out",
      "class": "File"
    }},
    "star_progress_log": {{
      "location": "{raw_data}/{uid}/{uid}.Log.progress.out",
      "class": "File"
    }},
    "star_sj_log": {{
      "location": "{raw_data}/{uid}/{uid}.SJ.out.tab",
      "class": "File"
    }},
    "rpkm_isoforms": {{
      "location": "{raw_data}/{uid}/{uid}.isoforms.csv",
      "class": "File"
    }}
  }}
"""

NARROW = """
  {{
    "macs2_narrow_peaks": {{
      "location": "{raw_data}/{uid}/{uid}_macs_peaks.narrowPeak",
      "class": "File"
    }},
    "macs2_peak_summits": {{
      "location": "{raw_data}/{uid}/{uid}_macs_summits.bed",
      "class": "File"
    }}
  }}
"""

BROAD = """
  {{
    "macs2_broad_peaks": {{
      "location": "{raw_data}/{uid}/{uid}_macs_peaks.broadPeak",
      "class": "File"
    }},
    "macs2_gapped_peak": {{
      "location": "{raw_data}/{uid}/{uid}_macs_peaks.gappedPeak",
      "class": "File"
    }}
  }}
"""

TRIM_SE = """
  {{
    "trim_report": {{
      "location": "{raw_data}/{uid}/{uid}.fastq_trimming_report.txt",
      "class": "File"
    }}
  }}
"""

TRIM_PE = """
  {{
    "trim_report_upstream": {{
      "location": "{raw_data}/{uid}/{uid}.fastq_trimming_report.txt",
      "class": "File"
    }},
    "trim_report_downstream": {{
      "location": "{raw_data}/{uid}/{uid}_2.fastq_trimming_report.txt",
      "class": "File"
    }}
  }}
"""

ADD_FIELDS = """
  {{
    "promoter": {outputs[promoter]}
  }}
"""

SE = """
  {{
    "fastx_statistics": {{
      "location": "{raw_data}/{uid}/{uid}.fastxstat",
      "class": "File"
    }}
  }}
"""

PE = """
  {{
    "fastx_statistics_upstream": {{
      "location": "{raw_data}/{uid}/{uid}.fastxstat",
      "class": "File"
    }},
    "fastx_statistics_downstream": {{
      "location": "{raw_data}/{uid}/{uid}_2.fastxstat",
      "class": "File"
    }}
  }}
"""

BIGWIG = """
  {{
    "bigwig": {{
      "location": "{raw_data}/{uid}/{uid}.bigWig",
      "class": "File"
    }}
  }}
"""

BIGWIG_DUTP = """
  {{
    "bigwig_upstream": {{
      "location": "{raw_data}/{uid}/{uid}_upstream.bigWig",
      "class": "File"
    }},
    "bigwig_downstream": {{
      "location": "{raw_data}/{uid}/{uid}_downstream.bigWig",
      "class": "File"
    }}
  }}
"""

BAM = """
  {{
    "bambai_pair": {{
      "location": "{raw_data}/{uid}/{uid}.bam",
      "class": "File",
      "secondaryFiles": [{{
        "location": "{raw_data}/{uid}/{uid}.bam.bai",
        "class": "File"
      }}]
    }}
  }}
"""

BAM_MITOCH = """
  {{
    "bam_merged": {{
      "location": "{raw_data}/{uid}/{uid}.bam",
      "class": "File"
    }}
  }}
"""

MODEL_R = """
  {{
    "macs2_moder_r": {{
      "location": "{raw_data}/{uid}/{uid}_macs_model.r",
      "class": "File"
    }}
  }}
"""


"""
+----+---------------------------------+----------------------------------+
| id | etype                           | workflow                         |
+----+---------------------------------+----------------------------------+
|  1 | DNA-Seq                         | chipseq-se.cwl                   |
|  2 | DNA-Seq pair                    | chipseq-pe.cwl                   |
|  3 | RNA-Seq                         | rnaseq-se.cwl                    |
|  4 | RNA-Seq pair                    | rnaseq-pe.cwl                    |
|  5 | RNA-Seq dUTP                    | rnaseq-se-dutp.cwl               |
|  6 | RNA-Seq dUTP pair               | rnaseq-pe-dutp.cwl               |
|  7 | RNA-Seq dUTP Mitochondrial      | rnaseq-se-dutp-mitochondrial.cwl |
|  8 | DNA-Seq Trim Galore             | trim-chipseq-se.cwl              |
|  9 | DNA-Seq pair Trim Galore        | trim-chipseq-pe.cwl              |
| 11 | RNA-Seq dUTP pair Mitochondrial | rnaseq-pe-dutp-mitochondrial.cwl |
+----+---------------------------------+----------------------------------+
"""


OUTPUT_TEMPLATES = {
    1: {
        "narrow": [COMMON, CHIP, BAM,             BIGWIG,              SE,     NARROW,        MODEL_R,          ADD_FIELDS],
        "broad":  [COMMON, CHIP, BAM,             BIGWIG,              SE,             BROAD, MODEL_R,          ADD_FIELDS]
    },
    2: {
        "narrow": [COMMON, CHIP, BAM,             BIGWIG,                  PE, NARROW,                          ADD_FIELDS],
        "broad":  [COMMON, CHIP, BAM,             BIGWIG,                  PE,         BROAD,                   ADD_FIELDS]
    },
    8: {
        "narrow": [COMMON, CHIP, BAM,             BIGWIG,              SE,     NARROW,        MODEL_R, TRIM_SE, ADD_FIELDS],
        "broad":  [COMMON, CHIP, BAM,             BIGWIG,              SE,             BROAD, MODEL_R, TRIM_SE, ADD_FIELDS]
    },
    9: {
        "narrow": [COMMON, CHIP, BAM,             BIGWIG,                  PE, NARROW,                 TRIM_PE, ADD_FIELDS],
        "broad":  [COMMON, CHIP, BAM,             BIGWIG,                  PE,         BROAD,          TRIM_PE, ADD_FIELDS]
    },
    3: {
        "narrow": [COMMON, RNA,  BAM,             BIGWIG,              SE,                                      ADD_FIELDS]
    },
    4: {
        "narrow": [COMMON, RNA,  BAM,             BIGWIG,                  PE,                                  ADD_FIELDS]
    },
    5: {
        "narrow": [COMMON, RNA,  BAM,                     BIGWIG_DUTP, SE,                                      ADD_FIELDS]
    },
    6: {
        "narrow": [COMMON, RNA,  BAM,                     BIGWIG_DUTP,     PE,                                  ADD_FIELDS]
    },
    7: {
        "narrow": [COMMON, RNA,       BAM_MITOCH,         BIGWIG_DUTP, SE,                                      ADD_FIELDS]
    },
    11: {
        "narrow": [COMMON, RNA,       BAM_MITOCH,         BIGWIG_DUTP,     PE,                                  ADD_FIELDS]
    }
}



"""Strategy pattern to run BaseUploader.execute depending on types of files to be uploaded"""
import os
import types
import logging
from biowardrobe_airflow_plugins.utils.connect import HookConnect

logger = logging.getLogger(__name__)


class Uploader:
    def __init__(self, func=None):
        if func is not None:
            self.execute = types.MethodType(func, self)


def process_results(upload_rules, uid, output_folder, clean=False):
    for func, filename in upload_rules.items():
        Uploader(UPLOAD_FUNCTIONS[func]).execute(uid,
                                                 os.path.join(output_folder, filename),
                                                 clean=clean)


def upload_bigbed(self, uid, filename, clean):
    logger.debug(f"Processing bigBed file: {filename}")
    connect_db = HookConnect()
    db_name = connect_db.fetchone(f"SELECT g.db FROM labdata l INNER JOIN genome g ON g.id=l.genome_id WHERE l.uid='{uid}'")['db']
    table_name = db_name + '.`' + str(uid).replace("-", "_") + '_senhncr_f_wtrack`'
    logger.debug(f"Drop table {table_name}")
    connect_db.execute(f"DROP TABLE IF EXISTS {table_name}")
    if not clean:
        logger.debug(f"Create table {table_name}")
        connect_db.execute(f"CREATE TABLE {table_name} (fileName VARCHAR(255) not NULL) ENGINE=MyISAM DEFAULT CHARSET=utf8")
        connect_db.execute(f"INSERT INTO {table_name} VALUES ('{filename}')")


UPLOAD_FUNCTIONS = {
    "upload_bigbed": upload_bigbed
}



#!/usr/bin/env python3
import logging
import decimal
import os
import re
from json import dumps, loads
from airflow.models import DagRun
from airflow.utils.state import State
from biowardrobe_airflow_plugins.utils.connect import HookConnect
from biowardrobe_airflow_plugins.utils.func import (norm_path, fill_template)
from biowardrobe_airflow_plugins.utils.upload import process_results


logger = logging.getLogger(__name__)


def get_data(uid):
    logger.debug(f"Collecting data for: {uid}")
    connect_db = HookConnect()
    settings = connect_db.get_settings()
    sql_query = f"""SELECT
                        l.uid                              as uid,
                        l.params                           as outputs,
                        l.control_id                       as control_uid,
                        COALESCE(l.trim5,0)                as trim5,
                        COALESCE(l.trim3,0)                as trim3,
                        COALESCE(l.fragmentsizeexp,0)      as exp_fragment_size,
                        COALESCE(l.fragmentsizeforceuse,0) as force_fragment_size,
                        COALESCE(l.rmdup,0)                as remove_duplicates,
                        e.etype                            as exp_type,
                        e.id                               as exp_type_id,
                        g.findex                           as genome_type, 
                        g.gsize                            as genome_size,
                        COALESCE(a.properties,0)           as broad_peak
                    FROM labdata l
                    INNER JOIN (experimenttype e, genome g) ON (e.id=l.experimenttype_id AND g.id=l.genome_id)
                    LEFT JOIN (antibody a) ON (l.antibody_id=a.id)
                    WHERE l.uid='{uid}'"""

    logger.debug(f"Running SQL query:\n{sql_query}")

    kwargs = connect_db.fetchone(sql_query)
    logger.debug(f"Collecting SQL query results:\n{kwargs}")
    kwargs = {key: (value if not isinstance(value, decimal.Decimal) else int(value)) for key, value in kwargs.items()}

    kwargs.update({
        "outputs": loads(kwargs['outputs']) if kwargs['outputs'] else {},
        "force_fragment_size": (int(kwargs['force_fragment_size']) == 1),
        "remove_duplicates": (int(kwargs['remove_duplicates']) == 1),
        "pair": ('pair' in kwargs['exp_type']),
        "dutp": ('dUTP' in kwargs['exp_type']),
        "broad_peak": (int(kwargs['broad_peak']) == 2),
        "raw_data": norm_path("/".join((settings['wardrobe'], settings['preliminary']))),
        "upload":   norm_path("/".join((settings['wardrobe'], settings['upload']))),
        "indices":  norm_path("/".join((settings['wardrobe'], settings['indices']))),
        "threads": settings['maxthreads'],
        "experimentsdb": settings['experimentsdb']
    })

    kwargs.update({"plugins": {plugin['workflow']: {'id':           plugin['id'],
                                                    'ptype':        plugin['ptype'],
                                                    'etype_id':     plugin['etype_id'],
                                                    'job':          fill_template(plugin['template'], kwargs),
                                                    'upload_rules': fill_template(plugin['upload_rules'], kwargs)}
                               for plugin in connect_db.fetchall("SELECT * FROM plugintype")
                               if kwargs['exp_type_id'] in loads(plugin['etype_id'])}})

    logger.debug(f"Collected data for: {uid}\n{dumps(kwargs, indent=4)}")
    return kwargs


def get_plugins(uid):
    logger.debug(f"Collecting plugins for: {uid}")
    connect_db = HookConnect()
    exp_type_id = connect_db.fetchone(f"""SELECT e.id FROM experimenttype e
                                          INNER JOIN (labdata l) ON (e.id=l.experimenttype_id)
                                          WHERE l.uid='{uid}'""")["id"]
    plugins = [plugin for plugin in connect_db.fetchall("SELECT id, ptype, workflow, etype_id FROM plugintype")
               if exp_type_id in loads(plugin["etype_id"])]
    return plugins


def get_active_plugins (uid):
    dag_ids = [os.path.splitext(plugin['workflow'])[0].replace(".", "_dot_") for plugin in get_plugins(uid)]
    return [{"dag_id": dag_id,
             "run_id": dagrun.run_id} for dag_id in dag_ids
                                          for dagrun in DagRun.find(dag_id, state=State.RUNNING)
                                              if uid in dagrun.run_id]


def clean_database(uid):
    for workflow, plugin_data in get_data(uid)['plugins'].items():
        process_results(upload_rules=plugin_data['upload_rules'],
                        uid=uid,
                        output_folder=plugin_data['job']['output_folder'],
                        clean=True)


def clean_files(uid, pattern):
    connect_db = HookConnect()
    settings = connect_db.get_settings()
    working_dir = norm_path("/".join((settings['wardrobe'], settings['preliminary'], uid)))
    for file in os.listdir(working_dir):
        if re.match(pattern, file):
            logger.info(f"Removing {os.path.join(working_dir, file)}")
            os.remove(os.path.join(working_dir, file))


def clean_workspace(uid, pattern):
    logger.debug(f"Cleaning workspace for: {uid}")
    clean_database(uid)
    clean_files(uid, pattern)




import os
import uuid
import logging
from datetime import datetime
from sqlalchemy.exc import SQLAlchemyError
from airflow import settings
from airflow.models import (BaseOperator, DagRun)
from biowardrobe_airflow_plugins.utils.analyze import (get_plugins, get_active_plugins, clean_workspace)


logger = logging.getLogger(__name__)


class BioWardrobePluginTrigger(BaseOperator):

    def __init__(self, *args, **kwargs):
        super(BioWardrobePluginTrigger, self).__init__(task_id=self.__class__.__name__, *args, **kwargs)

    def execute(self, context):
        uid = context['dag_run'].conf['uid']
        active_plugins = get_active_plugins(uid)
        if active_plugins:
            logger.info(f"""Active plugins found for {uid}""")
            for active_plugin in active_plugins:
                logger.info(f"""Skip trigger {active_plugin['dag_id']} because of {active_plugin['run_id']}""")
        else:
            clean_workspace(uid, f"{uid}_default_.*")
            for plugin in get_plugins(uid):
                try:
                    dag_id = os.path.splitext(plugin['workflow'])[0].replace(".", "_dot_")
                    run_id = "__".join(["trig", uid, str(uuid.uuid4())])
                    payload = {'uid': uid, 'run_id': run_id}
                    session = settings.Session()
                    dr = DagRun(dag_id=dag_id,
                                run_id=run_id,
                                conf=payload,
                                external_trigger=True)
                    session.add(dr)
                    session.commit()
                    session.close()
                    logger.info(f"""Trigger {plugin["ptype"]} ({dag_id}) plugin for {uid}:\nrun_id: {run_id}""")
                except SQLAlchemyError as err:
                    logger.error(f"""Failed to trigger {plugin["ptype"]} ({dag_id}) plugin for {uid}\n {err}""")



#!/usr/bin/env python3
from datetime import datetime
from airflow.models import DAG
from cwl_airflow_parser.cwldag import CWLDAG
from biowardrobe_airflow_plugins.operators.biowardrobepluginjobdispatcher import BioWardrobePluginJobDispatcher
from biowardrobe_airflow_plugins.operators.biowardrobepluginjobgatherer import BioWardrobePluginJobGatherer
from biowardrobe_airflow_plugins.operators.biowardrobeplugintrigger import BioWardrobePluginTrigger
from biowardrobe_airflow_plugins.utils.func import get_workflow


def create_plugin(workflow_name):
    dag = CWLDAG(default_args={'pool': 'biowardrobe_plugins'},
                 cwl_workflow = get_workflow(workflow_name))
    dag.create()
    dag.add(BioWardrobePluginJobDispatcher(dag=dag), to='top')
    dag.add(BioWardrobePluginJobGatherer(dag=dag), to='bottom')
    return dag


def create_plugin_trigger():
    dag = DAG(
        dag_id='biowardrobe_plugins',
        default_args = {'pool': 'biowardrobe_plugins',
                        'start_date': datetime.now()},
        schedule_interval=None
    )
    trigger = BioWardrobePluginTrigger(dag=dag)
    return dag













import os
from cwl_airflow_parser import CWLJobDispatcher
from biowardrobe_airflow_plugins.utils.analyze import get_data


class BioWardrobePluginJobDispatcher(CWLJobDispatcher):

    def __init__(self, *args, **kwargs):
        super(BioWardrobePluginJobDispatcher, self).__init__(task_id=self.__class__.__name__, *args, **kwargs)

    def execute(self, context):
        conf = context['dag_run'].conf
        try:
            return self.cwl_dispatch(conf['job'])
        except KeyError:
            workflow = os.path.basename(self.dag.default_args["cwl_workflow"])
            return self.cwl_dispatch(get_data(conf['uid'])['plugins'][workflow]['job'])




#! /usr/bin/env python3
import MySQLdb                                                 # TODO Do I need to add it into requirements in setup.py
import logging
from contextlib import closing
from airflow.hooks.mysql_hook import MySqlHook
from airflow.utils.db import merge_conn
from airflow import models
from sqlparse import split
from biowardrobe_airflow_plugins.utils.func import open_file


logger = logging.getLogger(__name__)


class Connect:

    def get_conn(self):
        pass

    def execute(self, sql, option=None):
        with closing(self.get_conn()) as connection:
            with closing(connection.cursor()) as cursor:
                cursor.execute(sql)
                connection.commit()
                if option == 1:
                    return cursor.fetchone()
                elif option == 2:
                    return cursor.fetchall()
                else:
                    return None

    def fetchone(self, sql):
        return self.execute(sql,1)

    def fetchall(self, sql):
        return self.execute(sql,2)

    def get_settings(self):
        return {row['key']: row['value'] for row in self.fetchall("SELECT * FROM settings")}

    def apply_patch(self, filename):
        logger.debug(f"Apply SQL patch: {filename}")
        with open(filename) as patch_stream:
            for sql_segment in split(patch_stream.read()):
                if sql_segment:
                    self.execute(sql_segment)


class DirectConnect(Connect):

    def __init__(self, config_file):
        self.config = [line for line in open_file(config_file) if not line.startswith("#")]

    def get_conn(self):
        conn_config = {
            "host": self.config[0],
            "user": self.config[1],
            "passwd": self.config[2],
            "db": self.config[3],
            "port": int(self.config[4]),
            "cursorclass": MySQLdb.cursors.DictCursor
        }
        conn = MySQLdb.connect(**conn_config)
        return conn


class HookConnect(Connect):

    CONNECTION_ID = "biowardrobe"

    def __init__(self, config_file = None):
        if config_file:
            self.config = [line for line in open_file(config_file) if not line.startswith("#")]
            merge_conn(
                models.Connection(
                    conn_id = self.CONNECTION_ID,
                    conn_type = 'mysql',
                    host = self.config[0],
                    login = self.config[1],
                    password = self.config[2],
                    schema = self.config[3],
                    extra = "{\"cursor\":\"dictcursor\"}"))
            self.get_conn()

    def get_conn(self):
        mysql = MySqlHook(mysql_conn_id=self.CONNECTION_ID)
        return mysql.get_conn()



#! /usr/bin/env python3
import os
import sys
import logging
import argparse
from typing import Text
from json import dumps, loads

from biowardrobe_airflow_plugins.utils.connect import (DirectConnect, HookConnect)
from biowardrobe_airflow_plugins.utils.func import (norm_path,
                                                    normalize_args,
                                                    get_files,
                                                    fill_template,
                                                    validate_locations,
                                                    add_details_to_outputs)
from biowardrobe_airflow_plugins.templates.outputs import OUTPUT_TEMPLATES

from airflow.settings import DAGS_FOLDER
from airflow.bin.cli import api_client


logger = logging.getLogger(__name__)


def get_parser():
    parser = argparse.ArgumentParser(description='BioWardrobe Airflow Plugins', add_help=True)
    parser.add_argument("-c", "--config", type=Text, help="Path to the BioWardrobe config file", default="/etc/wardrobe/wardrobe")
    return parser


def apply_patches(connect_db):
    current_dir = os.path.dirname(os.path.abspath(__file__))

    bw_patches = get_files(norm_path(os.path.join(current_dir, "sql_patches", "biowardrobe")), ".*sql$")
    gb_patches = get_files(norm_path(os.path.join(current_dir, "sql_patches", "genome_browser")), ".*sql$")

    if not connect_db.fetchone("SHOW TABLES LIKE 'plugintype'"):
        connect_db.apply_patch(bw_patches["plugintype_create.sql"])

    connect_db.apply_patch(bw_patches["plugintype_patch.sql"])

    for filename in gb_patches.values():
        try:
            connect_db.apply_patch(filename)
        except Exception:
            pass


def gen_outputs(connect_db):
    settings = connect_db.get_settings()
    raw_data = norm_path("/".join((settings['wardrobe'], settings['preliminary'])))
    sql_query = """SELECT
                         l.uid                    as uid,
                         l.params                 as outputs,
                         e.etype                  as exp_type,
                         e.id                     as exp_id,
                         COALESCE(a.properties,0) as peak_type
                   FROM  labdata l
                   INNER JOIN (experimenttype e) ON (e.id=l.experimenttype_id)
                   LEFT JOIN (antibody a) ON (l.antibody_id=a.id)
                   WHERE (l.params NOT LIKE '%file:%') AND
                         (l.deleted=0)                 AND
                         (l.libstatus=12)              AND
                         COALESCE(l.egroup_id,'')<>''  AND
                         COALESCE(l.name4browser,'')<>''"""

    for kwargs in connect_db.fetchall(sql_query):
        try:
            kwargs.update({"raw_data":   raw_data,
                           "peak_type": "broad" if int(kwargs['peak_type']) == 2 else "narrow",
                           "outputs":    loads(kwargs['outputs']) if kwargs['outputs'] and kwargs['outputs'] != "null" else {}})
            kwargs["outputs"]["promoter"] = kwargs["outputs"]["promoter"] if "promoter" in kwargs["outputs"] else 1000
            for template in OUTPUT_TEMPLATES[kwargs['exp_id']][kwargs['peak_type']]:
                kwargs["outputs"].update(fill_template(template, kwargs))
            list(validate_locations(kwargs))
            add_details_to_outputs(kwargs["outputs"])
            connect_db.execute(f"""UPDATE labdata SET params='{dumps(kwargs["outputs"])}' WHERE uid='{kwargs["uid"]}'""")
            logger.info(f"""Update params for {kwargs['uid']}\n {dumps(kwargs["outputs"], indent=4)}""")
        except Exception:
            logger.debug(f"Failed to updated params for {kwargs['uid']}")


def export_dag(template, filename):
    output_filename = os.path.abspath(os.path.join(DAGS_FOLDER, os.path.splitext(filename)[0]+".py"))
    with open(output_filename, 'w') as output_stream:
        output_stream.write(template.format(filename))


def create_dags():
    current_dir = os.path.join(os.path.dirname(os.path.abspath(__file__)), "cwls")
    template = u"""#!/usr/bin/env python3\nfrom airflow import DAG\nfrom biowardrobe_airflow_plugins import biowardrobe_plugin\ndag = biowardrobe_plugin("{}")"""
    for filename, path in get_files(current_dir).items():
        export_dag(template, filename)
    biowardrobe_plugin_trigger = u"#!/usr/bin/env python3\nfrom airflow import DAG\nfrom biowardrobe_airflow_plugins import biowardrobe_plugin_trigger\ndag = biowardrobe_plugin_trigger()"
    export_dag(biowardrobe_plugin_trigger, "biowardrobe_plugins")

def create_pools():
    try:
        pools = [api_client.get_pool(name="biowardrobe_plugins")]
    except Exception:
        api_client.create_pool(name="biowardrobe_plugins", slots=10, description="pool to run BioWardrobe plugins")


def create_connection(config):
    HookConnect(config)


def setup_airflow(config):
    create_connection(config)
    create_pools()
    create_dags()


def setup_biowardrobe(config):
    db_connection_handler = DirectConnect(config)
    apply_patches(db_connection_handler)
    gen_outputs(db_connection_handler)


def main(argsl=None):
    if argsl is None:
        argsl = sys.argv[1:]
    args,_ = get_parser().parse_known_args(argsl)
    args = normalize_args(args)

    setup_biowardrobe(args.config)
    setup_airflow(args.config)


if __name__ == "__main__":
    sys.exit(main(sys.argv[1:]))





import os
import re
import argparse
from functools import partial
from collections import OrderedDict
from json import loads
from cwltool.pathmapper import adjustFileObjs, adjustDirObjs, normalizeFilesDirs
from cwltool.process import compute_checksums
from cwltool.stdfsaccess import StdFsAccess
from schema_salad.ref_resolver import file_uri


def remove_not_set_inputs(job_object):
    job_object_filtered ={}
    for key, value in job_object.items():
        if complete_input(value):
            job_object_filtered[key] = value
    return job_object_filtered


def complete_input(item):
    monitor = {"found_none": False}
    recursive_check(item, monitor)
    return not monitor["found_none"]


def recursive_check(item, monitor):
    if item == 'None' or (isinstance(item, str) and 'None' in item):
        monitor["found_none"] = True
    elif isinstance(item, dict):
        dict((k, v) for k, v in item.items() if recursive_check(v, monitor))
    elif isinstance(item, list):
        list(v for v in item if recursive_check(v, monitor))


def fill_template(template, kwargs):
    job_object = remove_not_set_inputs(loads(template.replace('\n', ' ').format(**kwargs).
                                             replace("'True'", 'true').replace("'False'", 'false').
                                             replace('"True"', 'true').replace('"False"', 'false')))
    return OrderedDict(sorted(job_object.items()))


def norm_path(path):
    return os.path.abspath(os.path.normpath(os.path.normcase(path)))


def get_files(current_dir, filename_pattern=".*"):
    """Files with the identical basenames are overwritten"""
    files_dict = {}
    for root, dirs, files in os.walk(current_dir):
        files_dict.update(
            {filename: os.path.join(root, filename) for filename in files if re.match(filename_pattern, filename)}
        )
    return files_dict


def get_workflow(workflow_name):
    workflows_folder = norm_path(os.path.join(os.path.dirname(os.path.abspath(os.path.join(__file__, "../"))), "cwls"))
    return get_files(workflows_folder)[workflow_name]


def normalize_args(args, skip_list=[]):
    """Converts all relative path arguments to absolute ones relatively to the current working directory"""
    normalized_args = {}
    for key,value in args.__dict__.items():
        if key not in skip_list:
            normalized_args[key] = value if not value or os.path.isabs(value) else os.path.normpath(os.path.join(os.getcwd(), value))
        else:
            normalized_args[key]=value
    return argparse.Namespace (**normalized_args)


def open_file(filename):
    """Returns list of lines from the text file. \n at the end of the lines are trimmed. Empty lines are excluded"""
    lines = []
    with open(filename, 'r') as infile:
        for line in infile:
            if line.strip():
                lines.append(line.strip())
    return lines


def export_to_file (output_filename, data):
    with open(output_filename, 'w') as output_file:
        output_file.write(data)


def dict_find(dictionary, key):
    for k, v in dictionary.items():
        if k == key:
            yield v
        elif isinstance(v, dict):
            for result in dict_find(v, key):
                yield result
        elif isinstance(v, list):
            for d in v:
                for result in dict_find(d, key):
                    yield result


def validate_locations(dictionary, key="location"):
    for k in list(dictionary.keys()):
        if k == key:
            yield dictionary[k]
        elif isinstance(dictionary[k], dict):
            for result in validate_locations(dictionary[k], key):
                if not os.path.exists(result):
                    del dictionary[k]
        elif isinstance(dictionary[k], list):
            for d in dictionary[k]:
                for result in validate_locations(d, key):
                    if not os.path.exists(result):
                        dictionary[k].remove(d)


def expand_to_file_uri(job_obj):
    if "location" in job_obj:
        job_obj["location"] = file_uri(job_obj["location"])


def add_details_to_outputs(outputs):
    adjustFileObjs(outputs, expand_to_file_uri)
    adjustDirObjs(outputs, expand_to_file_uri)
    normalizeFilesDirs(outputs)
    adjustFileObjs(outputs, partial(compute_checksums, StdFsAccess("")))
    


import os
from cwl_airflow_parser import CWLJobGatherer
from biowardrobe_airflow_plugins.utils.analyze import get_data
from biowardrobe_airflow_plugins.utils.upload import process_results


class BioWardrobePluginJobGatherer(CWLJobGatherer):

    def __init__(self, *args, **kwargs):
        super(BioWardrobePluginJobGatherer, self).__init__(task_id=self.__class__.__name__, *args, **kwargs)

    def execute(self, context):
        job_result, promises = self.cwl_gather(context)
        data = get_data(promises['uid'])
        workflow = os.path.basename(self.dag.default_args["cwl_workflow"])
        process_results(upload_rules=data['plugins'][workflow]['upload_rules'],
                        uid=promises['uid'],
                        output_folder=self.output_folder)
        return job_result



#! /usr/bin/env python3

from setuptools import setup, find_packages
from os import path
from subprocess import check_output
from time import strftime, gmtime

GIT_VERSION_FILE = path.join('biowardrobe_airflow_plugins', '.git_version')


def get_git_tag():
    return check_output(['git', 'describe', '--contains'], universal_newlines=True).strip()


def get_git_timestamp():
    gitinfo = check_output(
        ['git', 'log', '--first-parent', '--max-count=1',
         '--format=format:%ct', '.']).strip()
    return strftime('%Y%m%d%H%M%S', gmtime(int(gitinfo)))


def get_version():
    '''
    Tries to get pachage version with following order:
    0. default version
    1. from git_version file - when installing from pip, this is the only source to get version
    2. from tag
    3. from commit timestamp
    Updates/creates git_version file with the package version
    :return: package version 
    '''
    version = "1.0.0"                                      # set default version
    try:
        with open(GIT_VERSION_FILE, 'r') as input_stream:  # try to get version info from file
            version = input_stream.read()
    except Exception:
        pass
    try:
        version = get_git_tag()                            # try to get version info from the closest tag
    except Exception:
        try:
            version = '1.0.' + get_git_timestamp()         # try to get version info from commit date
        except Exception:
            pass
    try:
        with open(GIT_VERSION_FILE, 'w') as output_stream: # save updated version to file (or the same)
            output_stream.write(version)
    except Exception:
        pass
    return version


setup(
    name='biowardrobe-airflow-plugins',
    description="Add plugin workflows to BioWardrobe",
    long_description=open(path.join(path.dirname(__file__), 'README.md')).read(),
    long_description_content_type="text/markdown",
    version=get_version(),
    url='https://github.com/Barski-lab/biowardrobe-airflow-plugins',
    download_url='https://github.com/Barski-lab/biowardrobe-airflow-plugins',
    author='Michael Kotliar',
    author_email='misha.kotliar@gmail.com',
    license = 'Apache-2.0',
    packages=find_packages(),
    install_requires=[
        'sqlparse',
        'cwl-airflow-parser',
        'mysqlclient>=1.3.6'
    ],
    zip_safe=False,
    entry_points={
        'console_scripts': [
            "biowardrobe-plugin-init=biowardrobe_airflow_plugins.init_plugin:main"
        ]
    },
    include_package_data=True,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Environment :: Other Environment',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'Intended Audience :: Healthcare Industry',
        'License :: OSI Approved :: Apache Software License',
        'Natural Language :: English',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: POSIX',
        'Operating System :: POSIX :: Linux',
        'Operating System :: OS Independent',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: Microsoft :: Windows :: Windows 10',
        'Operating System :: Microsoft :: Windows :: Windows 8.1',
        'Programming Language :: Python :: 3.6',
        'Topic :: Scientific/Engineering',
        'Topic :: Scientific/Engineering :: Bio-Informatics',
        'Topic :: Scientific/Engineering :: Chemistry',
        'Topic :: Scientific/Engineering :: Information Analysis',
        'Topic :: Scientific/Engineering :: Medical Science Apps.'
    ]
)
